part of datepicker;

class _PickerHeaderPainter extends CustomPainter {
  _PickerHeaderPainter(
      this.visibleDates,
      this.headerStyle,
      this.view,
      this.numberOfWeeksInView,
      this.monthFormat,
      this._datePickerTheme,
      this.isRtl,
      this._locale,
      this.enableMultiView,
      this.multiViewSpacing,
      this._hoverColor,
      this._hovering)
      : super(repaint: visibleDates);

  final DateRangePickerHeaderStyle headerStyle;
  final DateRangePickerView view;
  final int numberOfWeeksInView;
  final SfDateRangePickerThemeData _datePickerTheme;
  final bool isRtl;
  final String monthFormat;
  final bool _hovering;
  final bool enableMultiView;
  final double multiViewSpacing;
  final Color _hoverColor;
  ValueNotifier<List<DateTime>> visibleDates;
  String _headerText;
  TextPainter _textPainter;
  final Locale _locale;

  @override
  void paint(Canvas canvas, Size size) {
    canvas.clipRect(Rect.fromLTWH(0, 0, size.width, size.height));
    double _xPosition = 0;
    _textPainter = _textPainter ?? TextPainter();
    _textPainter.textDirection = TextDirection.ltr;
    _textPainter.textWidthBasis = TextWidthBasis.longestLine;
    _textPainter.maxLines = 1;

    _headerText = '';
    final double _width =
        enableMultiView && headerStyle.textAlign == TextAlign.center
            ? (size.width - multiViewSpacing) / 2
            : size.width;
    final int count =
        enableMultiView && headerStyle.textAlign == TextAlign.center ? 2 : 1;
    for (int j = 0; j < count; j++) {
      final int currentViewIndex = isRtl ? _getRtlIndex(count, j) : j;
      _xPosition = (currentViewIndex * _width) + 10;
      final String text = _getHeaderText(j);
      _headerText += j == 1 ? ' ' + text : text;
      TextStyle style =
          headerStyle.textStyle ?? _datePickerTheme.headerTextStyle;
      if (_hovering) {
        style = style.copyWith(color: _hoverColor);
      }

      final TextSpan span = TextSpan(text: text, style: style);
      _textPainter.text = span;

      if (headerStyle.textAlign == TextAlign.justify) {
        _textPainter.textAlign = headerStyle.textAlign;
      }

      _textPainter.layout(
          minWidth: 0,
          maxWidth: ((currentViewIndex + 1) * _width) - _xPosition > 0
              ? ((currentViewIndex + 1) * _width) - _xPosition
              : 0);

      if (headerStyle.textAlign == TextAlign.center) {
        _xPosition = (currentViewIndex * _width) +
            (currentViewIndex * multiViewSpacing) +
            (_width / 2) -
            (_textPainter.width / 2);
      } else if ((!isRtl &&
              (headerStyle.textAlign == TextAlign.right ||
                  headerStyle.textAlign == TextAlign.end)) ||
          (isRtl &&
              (headerStyle.textAlign == TextAlign.left ||
                  headerStyle.textAlign == TextAlign.start))) {
        _xPosition =
            ((currentViewIndex + 1) * _width) - _textPainter.width - _xPosition;
      }
      _textPainter.paint(canvas,
          Offset(_xPosition, size.height / 2 - _textPainter.height / 2));
    }
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    final _PickerHeaderPainter oldWidget = oldDelegate;
    return oldWidget.headerStyle != headerStyle ||
        oldWidget.isRtl != isRtl ||
        oldWidget.numberOfWeeksInView != numberOfWeeksInView ||
        oldWidget._locale != _locale ||
        oldWidget._datePickerTheme != _datePickerTheme ||
        (kIsWeb &&
            (oldWidget._hovering != _hovering ||
                oldWidget._hoverColor != _hoverColor));
  }

  String _getHeaderText(int index) {
    final List<DateTime> _dates = visibleDates.value;
    final int count = enableMultiView ? 2 : 1;
    final int datesCount = _dates.length ~/ count;
    final int startIndex = index * datesCount;
    final int endIndex = ((index + 1) * datesCount) - 1;
    final int middleIndex = startIndex + (datesCount ~/ 2);
    if (view == DateRangePickerView.month &&
        numberOfWeeksInView != 6 &&
        _dates[startIndex].month != _dates[endIndex].month) {
      final String monthTextFormat =
          monthFormat == null || monthFormat.isEmpty ? 'MMM' : monthFormat;
      int _endIndex = _dates.length - 1;
      if (enableMultiView && headerStyle.textAlign == TextAlign.center) {
        _endIndex = endIndex;
      }

      final String startText = DateFormat(monthTextFormat, _locale.toString())
              .format(_dates[startIndex])
              .toString() +
          ' ' +
          _dates[startIndex].year.toString();
      final String endText = DateFormat(monthTextFormat, _locale.toString())
              .format(_dates[_endIndex])
              .toString() +
          ' ' +
          _dates[_endIndex].year.toString();
      if (startText == endText) {
        return startText;
      }

      return startText + ' - ' + endText;
    } else if (view == DateRangePickerView.month) {
      final String monthTextFormat =
          monthFormat == null || monthFormat.isEmpty ? 'MMMM' : monthFormat;
      final String _text = DateFormat(monthTextFormat, _locale.toString())
              .format(_dates[middleIndex])
              .toString() +
          ' ' +
          _dates[middleIndex].year.toString();
      if (enableMultiView && headerStyle.textAlign != TextAlign.center) {
        return _text +
            ' - ' +
            DateFormat(monthTextFormat, _locale.toString())
                .format(_dates[datesCount + middleIndex])
                .toString() +
            ' ' +
            _dates[datesCount + middleIndex].year.toString();
      }

      return _text;
    } else if (view == DateRangePickerView.year) {
      final DateTime date = _dates[middleIndex];
      if (enableMultiView && headerStyle.textAlign != TextAlign.center) {
        return date.year.toString() +
            ' - ' +
            _dates[datesCount + middleIndex].year.toString();
      }

      return date.year.toString();
    } else if (view == DateRangePickerView.decade) {
      final int year = (_dates[middleIndex].year ~/ 10) * 10;
      if (enableMultiView && headerStyle.textAlign != TextAlign.center) {
        return year.toString() +
            ' - ' +
            (((_dates[datesCount + middleIndex].year ~/ 10) * 10) + 9)
                .toString();
      }

      return year.toString() + ' - ' + (year + 9).toString();
    } else if (view == DateRangePickerView.century) {
      final int year = (_dates[middleIndex].year ~/ 100) * 100;
      if (enableMultiView && headerStyle.textAlign != TextAlign.center) {
        return year.toString() +
            ' - ' +
            (((_dates[datesCount + middleIndex].year ~/ 100) * 100) + 99)
                .toString();
      }

      return year.toString() + ' - ' + (year + 99).toString();
    }

    return '';
  }

  /// overrides this property to build the semantics information which uses to
  /// return the required information for accessibility, need to return the list
  /// of custom painter semantics which contains the rect area and the semantics
  /// properties for accessibility
  @override
  SemanticsBuilderCallback get semanticsBuilder {
    return (Size size) {
      final dynamic rect = Offset.zero & size;
      return <CustomPainterSemantics>[
        CustomPainterSemantics(
          rect: rect,
          properties: SemanticsProperties(
            label: _headerText != null ? _headerText.replaceAll('-', 'to') : '',
            textDirection: TextDirection.ltr,
          ),
        ),
      ];
    };
  }

  @override
  bool shouldRebuildSemantics(CustomPainter oldDelegate) {
    return true;
  }
}
